{***************************************************}
{* 2-to-4 decoder unit for LogicGates6 application *}
{***************************************************}

unit decoder_2to4;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, ExtCtrls, StdCtrls, common;

type
  {*********}
  { TfDec24 }
  {*********}
  TfDec24 = class(TForm)
    stTitle: TStaticText;
    imGate1, imGate2, imGate3, imGate4, imInv1, imInv2: TImage;
    imSwitch0, imSwitch1, imSwitchE: TImage;
    shSwitch0a, shSwitch0b, shSwitch1a, shSwitch1b, shSwitchEa, shSwitchEb: TShape;
    shLedA0, shLedA1, shLedE, shLedY0, shLedY1, shLedY2, shLedY3: TShape;
    stA0, stA1, stE, stY0, stY1, stY2, stY3, stDetails, stBoolean: TStaticText;
    laIn, laOut, laE, laA0, laA1, laY0, laY1, laY2, laY3: TLabel;
    Label5, Label6, Label7, Label8, Label9: TLabel;
    Label10, Label11, Label12, Label13, Label14, Label15, Label16, Label17, Label18, Label19: TLabel;
    Label20, Label21, Label22, Label23, Label24, Label25, Label26, Label27, Label28, Label29: TLabel;
    Label3, Label30, Label31, Label32, Label33, Label34, Label35, Label36, Label37, Label38, Label39: TLabel;
    Label4, Label40, Label42, Label43, Label44, Label45, Label46, Label47: TLabel;
    Shape1, Shape10, Shape11, Shape12, Shape13, Shape14, Shape15, Shape16, Shape17, Shape18, Shape19: TShape;
    Shape20, Shape21, Shape22, Shape23, Shape24, Shape25, Shape26, Shape27, Shape28, Shape29: TShape;
    Shape3, Shape30, Shape31, Shape32, Shape33, Shape34, Shape35, Shape36, Shape37, Shape38, Shape39: TShape;
    Shape40, Shape41, Shape42, Shape43, Shape44, Shape45, Shape46, Shape47, Shape48, Shape49: TShape;
    Shape50, Shape51, Shape52, Shape53, Shape54, Shape55, Shape56, Shape57, Shape58, Shape59: TShape;
    Shape6, Shape60, Shape61, Shape62, Shape63, Shape64, Shape65, Shape66, Shape7, Shape8, Shape9: TShape;
    Memo1: TMemo;
    btClose: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure imSwitch0Click(Sender: TObject);
    procedure imSwitch1Click(Sender: TObject);
    procedure imSwitchEClick(Sender: TObject);
    procedure btCloseClick(Sender: TObject);
  private
    bEnable: Boolean;                                                          // Enable line status (variables)
    aInput: array[0..1] of Boolean;                                            // input lines status (variables)
    aOutput: array[0..3] of Boolean;                                           // output lines status (variables)
    SwitchEnable: TSwitch;                                                     // Enable line switch
    aSwitches: array[0..1] of TSwitch;                                         // input lines switches
    LedEnable: TLed;                                                           // Enable line LED
    aLedsInput: array[0..1] of TLed;                                           // input lines LEDs
    aLedsOutput: array[0..3] of TLed;                                          // output lines LEDs
  end;

var
  fDec24: TfDec24;

implementation

{$R *.lfm}

{*********}
{ TfDec24 }
{*********}

{ Application start: Init the LED and Switch records }

procedure TfDec24.FormCreate(Sender: TObject);

var
  I: Integer;

begin
  // Apply subscripts
  stY3.Caption := StringReplace(stY3.Caption, '3', SUB_Numbers[3], []); stY2.Caption := StringReplace(stY2.Caption, '2', SUB_Numbers[2], []);
  stY1.Caption := StringReplace(stY1.Caption, '1', SUB_Numbers[1], []); stY0.Caption := StringReplace(stY0.Caption, '0', SUB_Numbers[0], []);
  stA1.Caption := StringReplace(stA1.Caption, '1', SUB_Numbers[1], []); stA0.Caption := StringReplace(stA0.Caption, '0', SUB_Numbers[0], []);
  laY3.Caption := StringReplace(laY3.Caption, '3', SUB_Numbers[3], []); laY2.Caption := StringReplace(laY2.Caption, '2', SUB_Numbers[2], []);
  laY1.Caption := StringReplace(laY1.Caption, '1', SUB_Numbers[1], []); laY0.Caption := StringReplace(laY0.Caption, '0', SUB_Numbers[0], []);
  laA1.Caption := StringReplace(laA1.Caption, '1', SUB_Numbers[1], []); laA0.Caption := StringReplace(laA0.Caption, '0', SUB_Numbers[0], []);
  for I := 0 to 3 do begin
    stDetails.Caption := StringReplace(stDetails.Caption, 'Y' + IntToStr(I), 'Y' + SUB_Numbers[I], [rfReplaceAll]);
    stBoolean.Caption := StringReplace(stBoolean.Caption, IntToStr(I), SUB_Numbers[I], [rfReplaceAll]);
  end;
  // Create arrays with switches and LEds
  LedEnable.LedShape := shLedE;
  aLedsInput[0].LedShape := shLedA0; aLedsInput[1].LedShape := shLedA1;
  aLedsOutput[0].LedShape := shLedY0; aLedsOutput[1].LedShape := shLedY1;
  aLedsOutput[2].LedShape := shLedY2; aLedsOutput[3].LedShape := shLedY3;
  with SwitchEnable do begin
    SwitchShapeA := shSwitchEa; SwitchShapeB := shSwitchEb;
    SwitchShapeAOffPos := 465;                                                 // vertical position (Top property) of open switch
  end;
  with aSwitches[0] do begin
    SwitchShapeA := shSwitch0a; SwitchShapeB := shSwitch0b;
    SwitchShapeAOffPos := 214;
  end;
  with aSwitches[1] do begin
    SwitchShapeA := shSwitch1a; SwitchShapeB := shSwitch1b;
    SwitchShapeAOffPos := 89;
  end;
end;

{ Window show-up: Init the decoder circuit }

procedure TfDec24.FormActivate(Sender: TObject);

var
  I: Integer;

begin
  // Enable line Low
  SwitchOff(SwitchEnable); LedOff(LedEnable); bEnable := False;
  // All input lines Low
  for I := 0 to 1 do begin
    SwitchOff(aSwitches[I]);
    LedOff(aLedsInput[I]);
    aInput[I] := False;
  end;
  // All output lines Low
  for I := 0 to 3 do begin
    LedOff(aLedsOutput[I]);
    aOutput[I] := False;
  end;
end;

{ User click on Enable line switch: Set Enable line High or Low; do 2-to-4 decoder simulation }

procedure TfDec24.imSwitchEClick(Sender: TObject);

begin
  SwitchToggle(SwitchEnable); bEnable := SwitchEnable.SwitchOn; LedOnOff(bEnable, LedEnable);
  Decoder24(bEnable, aInput, aOutput, aLedsOutput);
end;

{ User click on input line switches: Set corr. input line High or Low; do 2-to-4 decoder simulation }

procedure TfDec24.imSwitch0Click(Sender: TObject);

begin
  SwitchOnOff(0, True, not aSwitches[0].SwitchOn, aSwitches, aLedsInput, aInput, False, True);
  Decoder24(bEnable, aInput, aOutput, aLedsOutput);
end;

procedure TfDec24.imSwitch1Click(Sender: TObject);

begin
  SwitchOnOff(1, True, not aSwitches[1].SwitchOn, aSwitches, aLedsInput, aInput, False, True);
  Decoder24(bEnable, aInput, aOutput, aLedsOutput);
end;

{Button "Close": Close the simulation window }

procedure TfDec24.btCloseClick(Sender: TObject);

begin
  Close;
end;

end.

