{******************************************************}
{* 1x4 demultiplexer unit for LogicGates6 application *}
{******************************************************}

unit demultiplexer_1x4;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, ExtCtrls, StdCtrls, Menus, common;

type
  {************}
  { TfDemult14 }
  {************}
  TfDemult14 = class(TForm)
    mMenu: TMainMenu;
    mSettings, mSettingsColored: TMenuItem;
    stTitle: TStaticText;
    imGate1, imGate2, imGate3, imGate4, imInv1, imInv2: TImage;
    imSwitch0, imSwitch1, imSwitchI: TImage;
    shSwitch0a, shSwitch0b, shSwitch1a, shSwitch1b, shSwitchIa, shSwitchIb: TShape;
    shLedS0, shLedS1, shLedI, shLedY0, shLedY1, shLedY2, shLedY3: TShape;
    stS0, stS1, stI, stY0, stY1, stY2, stY3, stDetails, stExpl, stBoolean: TStaticText;
    laSel, laOutput, laS0, laS1, laY0, laY1, laY2, laY3: TLabel;
    Label10, Label11, Label12, Label13, Label14, Label16, Label17, Label18, Label19: TLabel;
    Label20, Label21, Label22, Label23, Label24, Label25, Label26, Label27, Label28, Label29: TLabel;
    Label3, Label34, Label35, Label36, Label37, Label38, Label39, Label4, Label42: TLabel;
    Label43, Label44, Label45, Label46, Label47: TLabel;
    Shape1, Shape10, Shape11, Shape12, Shape13, Shape14, Shape15, Shape16, Shape17, Shape18, Shape19: TShape;
    Shape20, Shape21, Shape22, Shape23, Shape24, Shape25, Shape26, Shape27, Shape28, Shape29: TShape;
    Shape3, Shape30, Shape31, Shape32, Shape33, Shape34, Shape35, Shape36, Shape37, Shape38, Shape39: TShape;
    Shape40, Shape41, Shape42, Shape43, Shape44, Shape45, Shape46, Shape47, Shape48, Shape49: TShape;
    Shape50, Shape51, Shape52, Shape53, Shape54, Shape55, Shape56, Shape57, Shape58, Shape59: TShape;
    Shape6, Shape60, Shape61, Shape62, Shape63, Shape64, Shape65, Shape66: TShape;
    Shape67, Shape7, Shape8, Shape9: TShape;
    Memo1: TMemo;
    btClose: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure mSettingsColoredClick(Sender: TObject);
    procedure imSwitch0Click(Sender: TObject);
    procedure imSwitch1Click(Sender: TObject);
    procedure imSwitchIClick(Sender: TObject);
    procedure btCloseClick(Sender: TObject);
  private
    aSelection: array[0..1] of Boolean;                                        // selection lines status (variables)
    bInput: Boolean;                                                           // input line status (variables)
    aOutput: array[0..3] of Boolean;                                           // output lines status (variables)
    iLine: Integer;                                                            // output line actually selected
    aSwitches: array[0..1] of TSwitch;                                         // selection lines switches
    SwitchInput: TSwitch;                                                      // input line switch
    aLedsSelection: array[0..1] of TLed;                                       // selection lines LEDs
    LedInput: TLed;                                                            // input line LED
    aLedsOutput: array[0..3] of TLed;                                          // output lines LEDs
    bColoredLeds: Boolean;
  end;

const
  // Input LEDs default color
  InputLedDefaultColor: TColor = clLime;

var
  fDemult14: TfDemult14;

implementation

{$R *.lfm}

{************}
{ TfDemult14 }
{************}

{ Application start: Init the LED and Switch records }

procedure TfDemult14.FormCreate(Sender: TObject);

var
  I: Integer;

begin
  // Apply subscripts
  stS1.Caption := StringReplace(stS1.Caption, '1', SUB_Numbers[1], []); stS0.Caption := StringReplace(stS0.Caption, '0', SUB_Numbers[0], []);
  laY3.Caption := StringReplace(laY3.Caption, '3', SUB_Numbers[3], []); laY2.Caption := StringReplace(laY2.Caption, '2', SUB_Numbers[2], []);
  laY1.Caption := StringReplace(laY1.Caption, '1', SUB_Numbers[1], []); laY0.Caption := StringReplace(laY0.Caption, '0', SUB_Numbers[0], []);
  laS1.Caption := StringReplace(laS1.Caption, '1', SUB_Numbers[1], []); laS0.Caption := StringReplace(laS0.Caption, '0', SUB_Numbers[0], []);
  for I := 0 to 3 do begin
    stDetails.Caption := StringReplace(stDetails.Caption, 'Y' + IntToStr(I), 'Y' + SUB_Numbers[I], [rfReplaceAll]);
    stDetails.Caption := StringReplace(stDetails.Caption, 'S' + IntToStr(I), 'S' + SUB_Numbers[I], [rfReplaceAll]);
    stExpl.Caption := StringReplace(stExpl.Caption, IntToStr(I), SUB_Numbers[I], [rfReplaceAll]);
    stBoolean.Caption := StringReplace(stBoolean.Caption, IntToStr(I), SUB_Numbers[I], [rfReplaceAll]);
  end;
  // Create arrays with switches and LEds
  aLedsSelection[0].LedShape := shLedS0; aLedsSelection[1].LedShape := shLedS1;
  LedInput.LedShape := shLedI;
  aLedsOutput[0].LedShape := shLedY0; aLedsOutput[1].LedShape := shLedY1;
  aLedsOutput[2].LedShape := shLedY2; aLedsOutput[3].LedShape := shLedY3;
  with aSwitches[0] do begin
    SwitchShapeA := shSwitch0a; SwitchShapeB := shSwitch0b;
    SwitchShapeAOffPos := 214;                                                 // vertical position (Top property) of open switch
  end;
  with aSwitches[1] do begin
    SwitchShapeA := shSwitch1a; SwitchShapeB := shSwitch1b;
    SwitchShapeAOffPos := 89;
  end;
  with SwitchInput do begin
    SwitchShapeA := shSwitchIa; SwitchShapeB := shSwitchIb;
    SwitchShapeAOffPos := 340;
  end;
  // No colored LEDs at application start
  bColoredLeds := False;
end;

{ Window show-up: Init the demultiplexer circuit }

procedure TfDemult14.FormActivate(Sender: TObject);

var
  I: Integer;

begin
  // All selection lines Low
  for I := 0 to 1 do begin
    SwitchOff(aSwitches[I]);
    LedOff(aLedsSelection[I]);
    aSelection[I] := False;
  end;
  // Input LED color
  SetLedColor(iLine, InputLedDefaultColor, bColoredLeds, LedInput);
  // Input line Low
  SwitchOff(SwitchInput); LedOff(LedInput); bInput := False;
  // Output LED color
  SetLedColors(clYellow, bColoredLeds, aLedsOutput);
  // All output lines Low
  for I := 0 to 3 do begin
    LedOff(aLedsOutput[I]);
    aOutput[I] := False;
  end;
  // Output line actually selected
  iLine := 0;
end;

{ Menu item "Settings > Use colored output LEDs": Toggle colored/not colored output LEDs }

procedure TfDemult14.mSettingsColoredClick(Sender: TObject);

begin
  if mSettingsColored.Checked then
    mSettingsColored.Checked := False
  else
    mSettingsColored.Checked := True;
  bColoredLeds := mSettingsColored.Checked;
  // Output lines LEDs color; use this color also for input LED
  // This is a simple way to show, which output is actually selected and thus connected to the input
  SetLedColor(iLine, InputLedDefaultColor, bColoredLeds, LedInput);
  SetLedColors(clYellow, bColoredLeds, aLedsOutput);
end;

{ User click on selection line switches: Set corr. selection line High or Low; do 1x4 demultiplexer simulation }

procedure TfDemult14.imSwitch0Click(Sender: TObject);

begin
  SwitchOnOff(0, True, not aSwitches[0].SwitchOn, aSwitches, aLedsSelection, aSelection, False, True);
  DeMultiplexer14(aSelection, bInput, iLine, aOutput, aLedsOutput);
  SetLedColor(iLine, InputLedDefaultColor, bColoredLeds, LedInput);
end;

procedure TfDemult14.imSwitch1Click(Sender: TObject);

begin
  SwitchOnOff(1, True, not aSwitches[1].SwitchOn, aSwitches, aLedsSelection, aSelection, False, True);
  DeMultiplexer14(aSelection, bInput, iLine, aOutput, aLedsOutput);
  SetLedColor(iLine, InputLedDefaultColor, bColoredLeds, LedInput);
end;

{ User click on input line switch: Set input line High or Low; do 1x4 demultiplexer simulation }

procedure TfDemult14.imSwitchIClick(Sender: TObject);

begin
  SwitchToggle(SwitchInput); bInput := SwitchInput.SwitchOn;  LedOnOff(bInput, LedInput);
  DeMultiplexer14(aSelection, bInput, iLine, aOutput, aLedsOutput);
  SetLedColor(iLine, InputLedDefaultColor, bColoredLeds, LedInput);
end;

{Button "Close": Close the simulation window }

procedure TfDemult14.btCloseClick(Sender: TObject);

begin
  Close;
end;

end.

