{************************************************************}
{* 4-to-2 priority encoder unit for LogicGates6 application *}
{************************************************************}

unit encoder_4to2pri;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, ExtCtrls, StdCtrls, common;

type
  {**********}
  { TfEnc42p }
  {**********}
  TfEnc42p = class(TForm)
    stTitle: TStaticText;
    imGate1, imGate2, imGate3, imGate4, imInv: TImage;
    imSwitch0, imSwitch1, imSwitch2, imSwitch3: TImage;
    shSwitch0a, shSwitch0b, shSwitch1a, shSwitch1b: TShape;
    shSwitch2a, shSwitch2b, shSwitch3a, shSwitch3b: TShape;
    shLedY0, shLedY1, shLedY2, shLedY3, shLedA0, shLedA1, shLedV: TShape;
    stY0, stY1, stY2, stY3, stA0, stA1, stV: TStaticText;
    stDetails, stBoolean: TStaticText;
    laIn, laOut, laY0, laY1, laY2, laY3, laA0, laA1, laV: TLabel;
    Label1, Label10, Label11, Label12, Label13, Label14, Label15, Label16, Label17, Label18, Label19: TLabel;
    Label2, Label20, Label21, Label22, Label23, Label24, Label25, Label26, Label27, Label28, Label29: TLabel;
    Label3, Label30, Label31, Label32, Label33, Label34, Label35, Label36, Label37, Label38, Label39: TLabel;
    Label4, Label40, Label41, Label42, Label43, Label5, Label6, Label7, Label8, Label9: TLabel;
    Shape1, Shape10, Shape11, Shape12, Shape13, Shape14, Shape15, Shape16, Shape17, Shape18, Shape19: TShape;
    Shape2, Shape20, Shape21, Shape22, Shape23, Shape24, Shape25, Shape26, Shape27, Shape28, Shape29: TShape;
    Shape3, Shape30, Shape31, Shape32, Shape33, Shape34, Shape35, Shape36, Shape37, Shape38, Shape39: TShape;
    Shape4, Shape40, Shape41, Shape42, Shape43, Shape44, Shape45, Shape46, Shape47, Shape48, Shape49: TShape;
    Shape5, Shape50, Shape51, Shape52, Shape53, Shape54, Shape55, Shape56, Shape6, Shape61, Shape7, Shape8, Shape9: TShape;
    Memo1: TMemo;
    btClose: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure imSwitch0Click(Sender: TObject);
    procedure imSwitch1Click(Sender: TObject);
    procedure imSwitch2Click(Sender: TObject);
    procedure imSwitch3Click(Sender: TObject);
    procedure btCloseClick(Sender: TObject);
  private
    aInput: array[0..3] of Boolean;                                            // input lines status (variables)
    aOutput: array[0..1] of Boolean;                                           // output lines status (variables)
    Validity: Boolean;                                                         // Validity line status (variables)
    aSwitches: array[0..3] of TSwitch;                                         // input lines switches
    aLedsInput: array[0..3] of TLed;                                           // input lines LEDs
    aLedsOutput: array[0..1] of TLed;                                          // output lines LEDs
    LedValidity: TLed;                                                         // Validity line LED
  end;

var
  fEnc42p: TfEnc42p;

implementation

{$R *.lfm}

{**********}
{ TfEnc42p }
{**********}

{ Application start: Init the LED and Switch records }

procedure TfEnc42p.FormCreate(Sender: TObject);

var
  I: Integer;

begin
  // Apply subscripts
  stY3.Caption := StringReplace(stY3.Caption, '3', SUB_Numbers[3], []); stY2.Caption := StringReplace(stY2.Caption, '2', SUB_Numbers[2], []);
  stY1.Caption := StringReplace(stY1.Caption, '1', SUB_Numbers[1], []); stY0.Caption := StringReplace(stY0.Caption, '0', SUB_Numbers[0], []);
  stA1.Caption := StringReplace(stA1.Caption, '1', SUB_Numbers[1], []); stA0.Caption := StringReplace(stA0.Caption, '0', SUB_Numbers[0], []);
  laY3.Caption := StringReplace(laY3.Caption, '3', SUB_Numbers[3], []); laY2.Caption := StringReplace(laY2.Caption, '2', SUB_Numbers[2], []);
  laY1.Caption := StringReplace(laY1.Caption, '1', SUB_Numbers[1], []); laY0.Caption := StringReplace(laY0.Caption, '0', SUB_Numbers[0], []);
  laA1.Caption := StringReplace(laA1.Caption, '1', SUB_Numbers[1], []); laA0.Caption := StringReplace(laA0.Caption, '0', SUB_Numbers[0], []);
  for I := 0 to 3 do begin
    stDetails.Caption := StringReplace(stDetails.Caption, 'Y' + IntToStr(I), 'Y' + SUB_Numbers[I], [rfReplaceAll]);
    stBoolean.Caption := StringReplace(stBoolean.Caption, IntToStr(I), SUB_Numbers[I], [rfReplaceAll]);
  end;
  // Create arrays with switches and LEds
  aLedsInput[0].LedShape := shLedY0; aLedsInput[1].LedShape := shLedY1;
  aLedsInput[2].LedShape := shLedY2; aLedsInput[3].LedShape := shLedY3;
  aLedsOutput[0].LedShape := shLedA0; aLedsOutput[1].LedShape := shLedA1;
  LedValidity.LedShape := shLedV;
  with aSwitches[0] do begin
    SwitchShapeA := shSwitch0a; SwitchShapeB := shSwitch0b;
    SwitchShapeAOffPos := 465;                                                 // vertical position (Top property) of open switch
  end;
  with aSwitches[1] do begin
    SwitchShapeA := shSwitch1a; SwitchShapeB := shSwitch1b;
    SwitchShapeAOffPos := 344;
  end;
  with aSwitches[2] do begin
    SwitchShapeA := shSwitch2a; SwitchShapeB := shSwitch2b;
    SwitchShapeAOffPos := 214;
  end;
  with aSwitches[3] do begin
    SwitchShapeA := shSwitch3a; SwitchShapeB := shSwitch3b;
    SwitchShapeAOffPos := 96;
  end;
end;

{ Window show-up: Init the priority encoder circuit }

procedure TfEnc42p.FormActivate(Sender: TObject);

var
  I: Integer;

begin
  // All input lines Low
  for I := 0 to 3 do begin
    SwitchOff(aSwitches[I]);
    LedOff(aLedsInput[I]);
    aInput[I] := False;
  end;
  // All output lines Low
  for I := 0 to 1 do begin
    LedOff(aLedsOutput[I]);
    aOutput[I] := False;
  end;
  // Validity line Low
  LedOff(LedValidity); Validity := False;
end;

{ User click on switches: Set corr. input line High or Low; do 4-to-2 priority encoder simulation }

procedure TfEnc42p.imSwitch0Click(Sender: TObject);

begin
  SwitchOnOff(0, True, not aSwitches[0].SwitchOn, aSwitches, aLedsInput, aInput, False, True);
  Encoder42Pri(aInput, aOutput, aLedsOutput, Validity, LedValidity);
end;

procedure TfEnc42p.imSwitch1Click(Sender: TObject);

begin
  SwitchOnOff(1, True, not aSwitches[1].SwitchOn, aSwitches, aLedsInput, aInput, False, True);
  Encoder42Pri(aInput, aOutput, aLedsOutput, Validity, LedValidity);
end;

procedure TfEnc42p.imSwitch2Click(Sender: TObject);

begin
  SwitchOnOff(2, True, not aSwitches[2].SwitchOn, aSwitches, aLedsInput, aInput, False, True);
  Encoder42Pri(aInput, aOutput, aLedsOutput, Validity, LedValidity);
end;

procedure TfEnc42p.imSwitch3Click(Sender: TObject);

begin
  SwitchOnOff(3, True, not aSwitches[3].SwitchOn, aSwitches, aLedsInput, aInput, False, True);
  Encoder42Pri(aInput, aOutput, aLedsOutput, Validity, LedValidity);
end;

{Button "Close": Close the simulation window }

procedure TfEnc42p.btCloseClick(Sender: TObject);

begin
  Close;
end;

end.

